@@ -15,23 +15,29 @@
 # This provides a simple database table interface built on top of
 # the Python BerkeleyDB 3 interface.
 #
-_cvsid = '$Id: dbtables.py 33462 2003-07-09 04:45:59Z greg $'
+_cvsid = '$Id: dbtables.py 58760 2007-11-01 21:22:40Z gregory.p.smith $'
 
 import re
 import sys
 import copy
-import xdrlib
-import whrandom
+import struct
+import random
 from types import ListType, StringType
 import cPickle as pickle
 
 try:
+    # For Pythons w/distutils pybsddb
+    from bsddb3.db import *
+except ImportError:
     # For Python 2.3
     from bsddb.db import *
-except ImportError:
-    # For earlier Pythons w/distutils pybsddb
-    from bsddb3.db import *
 
+# XXX(nnorwitz): is this correct? DBIncompleteError is conditional in _bsddb.c
+try:
+    DBIncompleteError
+except NameError:
+    class DBIncompleteError(Exception):
+        pass
 
 class TableDBError(StandardError):
     pass
@@ -131,7 +137,8 @@ def contains_metastrings(s) :
 class bsdTableDB :
     def __init__(self, filename, dbhome, create=0, truncate=0, mode=0600,
                  recover=0, dbflags=0):
-        """bsdTableDB.open(filename, dbhome, create=0, truncate=0, mode=0600)
+        """bsdTableDB(filename, dbhome, create=0, truncate=0, mode=0600)
+
         Open database name in the dbhome BerkeleyDB directory.
         Use keyword arguments when calling this constructor.
         """
@@ -206,7 +213,7 @@ class bsdTableDB :
         try:
             key, data = cur.first()
             while 1:
-                print `{key: data}`
+                print repr({key: data})
                 next = cur.next()
                 if next:
                     key, data = next
@@ -218,7 +225,8 @@ class bsdTableDB :
 
 
     def CreateTable(self, table, columns):
-        """CreateTable(table, columns) - Create a new table in the database
+        """CreateTable(table, columns) - Create a new table in the database.
+
         raises TableDBError if it already exists or for other DB errors.
         """
         assert isinstance(columns, ListType)
@@ -286,7 +294,8 @@ class bsdTableDB :
     def CreateOrExtendTable(self, table, columns):
         """CreateOrExtendTable(table, columns)
 
-        - Create a new table in the database.
+        Create a new table in the database.
+
         If a table of this name already exists, extend it to have any
         additional columns present in the given list as well as
         all of its current columns.
@@ -341,9 +350,9 @@ class bsdTableDB :
         try:
             tcolpickles = self.db.get(_columns_key(table))
         except DBNotFoundError:
-            raise TableDBError, "unknown table: " + `table`
+            raise TableDBError, "unknown table: %r" % (table,)
         if not tcolpickles:
-            raise TableDBError, "unknown table: " + `table`
+            raise TableDBError, "unknown table: %r" % (table,)
         self.__tablecolumns[table] = pickle.loads(tcolpickles)
 
     def __new_rowid(self, table, txn) :
@@ -351,12 +360,12 @@ class bsdTableDB :
         unique = 0
         while not unique:
             # Generate a random 64-bit row ID string
-            # (note: this code has <64 bits of randomness
+            # (note: might have <64 bits of true randomness
             # but it's plenty for our database id needs!)
-            p = xdrlib.Packer()
-            p.pack_int(int(whrandom.random()*2147483647))
-            p.pack_int(int(whrandom.random()*2147483647))
-            newid = p.get_buffer()
+            blist = []
+            for x in xrange(_rowid_str_len):
+                blist.append(random.randint(0,255))
+            newid = struct.pack('B'*_rowid_str_len, *blist)
 
             # Guarantee uniqueness by adding this key to the database
             try:
@@ -384,7 +393,7 @@ class bsdTableDB :
                 self.__load_column_info(table)
             for column in rowdict.keys() :
                 if not self.__tablecolumns[table].count(column):
-                    raise TableDBError, "unknown column: "+`column`
+                    raise TableDBError, "unknown column: %r" % (column,)
 
             # get a unique row identifier for this row
             txn = self.env.txn_begin()
@@ -411,14 +420,15 @@ class bsdTableDB :
 
 
     def Modify(self, table, conditions={}, mappings={}):
-        """Modify(table, conditions) - Modify in rows matching 'conditions'
-        using mapping functions in 'mappings'
-        * conditions is a dictionary keyed on column names
-        containing condition functions expecting the data string as an
-        argument and returning a boolean.
-        * mappings is a dictionary keyed on column names containint condition
-        functions expecting the data string as an argument and returning the
-        new string for that column.
+        """Modify(table, conditions={}, mappings={}) - Modify items in rows matching 'conditions' using mapping functions in 'mappings'
+
+        * table - the table name
+        * conditions - a dictionary keyed on column names containing
+          a condition callable expecting the data string as an
+          argument and returning a boolean.
+        * mappings - a dictionary keyed on column names containing a
+          condition callable expecting the data string as an argument and
+          returning the new string for that column.
         """
         try:
             matching_rowids = self.__Select(table, [], conditions)
@@ -434,7 +444,7 @@ class bsdTableDB :
                         try:
                             dataitem = self.db.get(
                                 _data_key(table, column, rowid),
-                                txn)
+                                txn=txn)
                             self.db.delete(
                                 _data_key(table, column, rowid),
                                 txn)
@@ -450,7 +460,8 @@ class bsdTableDB :
                         txn.commit()
                         txn = None
 
-                except DBError, dberror:
+                # catch all exceptions here since we call unknown callables
+                except:
                     if txn:
                         txn.abort()
                     raise
@@ -461,9 +472,10 @@ class bsdTableDB :
     def Delete(self, table, conditions={}):
         """Delete(table, conditions) - Delete items matching the given
         conditions from the table.
-        * conditions is a dictionary keyed on column names
-        containing condition functions expecting the data string as an
-        argument and returning a boolean.
+
+        * conditions - a dictionary keyed on column names containing
+          condition functions expecting the data string as an
+          argument and returning a boolean.
         """
         try:
             matching_rowids = self.__Select(table, [], conditions)
@@ -499,11 +511,12 @@ class bsdTableDB :
 
 
     def Select(self, table, columns, conditions={}):
-        """Select(table, conditions) - retrieve specific row data
+        """Select(table, columns, conditions) - retrieve specific row data
         Returns a list of row column->value mapping dictionaries.
-        * columns is a list of which column data to return.  If
+
+        * columns - a list of which column data to return.  If
           columns is None, all columns will be returned.
-        * conditions is a dictionary keyed on column names
+        * conditions - a dictionary keyed on column names
           containing callable conditions expecting the data string as an
           argument and returning a boolean.
         """
@@ -535,7 +548,7 @@ class bsdTableDB :
             columns = self.tablecolumns[table]
         for column in (columns + conditions.keys()):
             if not self.__tablecolumns[table].count(column):
-                raise TableDBError, "unknown column: "+`column`
+                raise TableDBError, "unknown column: %r" % (column,)
 
         # keyed on rows that match so far, containings dicts keyed on
         # column names containing the data for that row and column.
