@@ -4,7 +4,7 @@ Miscellaneous utility functions -- anything that doesn't fit into
 one of the other *util.py modules.
 """
 
-__revision__ = "$Id: util.py 52231 2006-10-08 17:41:25Z ronald.oussoren $"
+__revision__ = "$Id: util.py 59116 2007-11-22 10:14:26Z ronald.oussoren $"
 
 import sys, os, string, re
 from distutils.errors import DistutilsPlatformError
@@ -68,15 +68,15 @@ def get_platform ():
         if m:
             release = m.group()
     elif osname[:6] == "darwin":
-        # 
-        # For our purposes, we'll assume that the system version from 
+        #
+        # For our purposes, we'll assume that the system version from
         # distutils' perspective is what MACOSX_DEPLOYMENT_TARGET is set
         # to. This makes the compatibility story a bit more sane because the
         # machine is going to compile and link as if it were
         # MACOSX_DEPLOYMENT_TARGET.
         from distutils.sysconfig import get_config_vars
         cfgvars = get_config_vars()
-                
+
         macver = os.environ.get('MACOSX_DEPLOYMENT_TARGET')
         if not macver:
             macver = cfgvars.get('MACOSX_DEPLOYMENT_TARGET')
@@ -86,7 +86,7 @@ def get_platform ():
             # way to get the system version (see the documentation for
             # the Gestalt Manager)
             try:
-               f = open('/System/Library/CoreServices/SystemVersion.plist')
+                f = open('/System/Library/CoreServices/SystemVersion.plist')
             except IOError:
                 # We're on a plain darwin box, fall back to the default
                 # behaviour.
@@ -99,22 +99,21 @@ def get_platform ():
                 if m is not None:
                     macver = '.'.join(m.group(1).split('.')[:2])
                 # else: fall back to the default behaviour
-    
+
         if macver:
             from distutils.sysconfig import get_config_vars
             release = macver
-            osname = 'macosx'
-            platver = os.uname()[2]
-            osmajor = int(platver.split('.')[0])
-            
-            if osmajor >= 8 and \
+            osname = "macosx"
+
+
+            if (release + '.') >= '10.4.' and \
                     get_config_vars().get('UNIVERSALSDK', '').strip():
                 # The universal build will build fat binaries, but not on
                 # systems before 10.4
                 machine = 'fat'
-        
+
             elif machine in ('PowerPC', 'Power_Macintosh'):
-                # Pick a sane name for the PPC architecture
+                # Pick a sane name for the PPC architecture.
                 machine = 'ppc'
 
     return "%s-%s-%s" % (osname, release, machine)
